c This routine read the file with filters' transmission functions and
c return the characteristics of single and composite filters
c k = number of filters to be convolved (1: single filter, max=10)
c kfil(k) = number of filters in filtresfile to be convolved 
c wleff = effective wavelength of the filter defined as 
c      integral(R(lambda)*lambda dlambda)/integral(R(lambda) dlambda)
c surf = surface of the filter defined as 
c      integral(R(lambda)/Rmax dlambda)
c bp = bandpass of the filter defined as 
c      2*SQRT[integral(R(lambda)*(lambda-wleff)^2 dlambda)/
c             integral(R(lambda) dlambda)]
c iband = type of transmission function: 0. energy; 1. photons
c ----------------------------------------------------------------------
      SUBROUTINE read_filt(nconvol,kfil,wleff,surf,bandpass,iband)
      IMPLICIT NONE
      INTEGER i,j,k,l,kfil,np,npp,mxtyp_bc,mxtyp_as,mxtyp_pg,mxvega,
     &     mxwl_bc,mxwl_as,mxwl_pg,mxwl,mxage_bc,mxage_pg,mxage,
     &     iband,ifilt,mxwf,mxx,mxfil,mxafil,mxlyf,mxred,mxrp,nconvol,
     &     npc,it
c mxx  = max number of points of filter's transmission function in input
      PARAMETER (mxx=100000)
      INCLUDE 'dimensionz.dec'
      REAL*8 wleff,surf,conwl,conwl2,r,rmax,aux,bandpass,trapz1,rlin,
     &     wh,res,whc,resc,dx,dlambda,wh1,wh2,res1,res2,rr1,rr2,fwhm,
     &     wlmin,wlmax
      CHARACTER filtresfile*160,line*160
      DIMENSION wh(mxx),res(mxx),wh1(mxx),res1(mxx),wh2(mxx),res2(mxx)
      DIMENSION kfil(10),npc(10),whc(10,mxx),resc(10,mxx),aux(mxx)
      EXTERNAL trapz1,rlin
      COMMON/compose/ifilt,np(mxfil+mxafil)
      COMMON/rfilter/r(mxwf,mxfil+mxafil,2)
      COMMON/filterres/filtresfile
c ----------------------------------------------------------------------
c first read the given filter from the file with response functions
c (replace subroutine filter0) and store it in the r array
c NB: the counter ifilt is updated in the main programme and passed 
c     through the common
c
c determine the type of file from the first non empty line of the file:
c it=0: old format; it=1: new format
      CLOSE(81)
      OPEN(81,FILE=filtresfile,STATUS='old')
      it=0                      ! assume old format
 101  READ(81,'(a)',ERR=102,END=102) line
      IF (line.eq.' ') GO TO 101
      IF (line(1:1).eq.'#') it=1
      CLOSE(81)

c simple case: no filters convolution
      IF (nconvol.eq.1) THEN
         CLOSE(81)
         OPEN(81,FILE=filtresfile,STATUS='old')
c start reading filters' file
 103     READ(81,'(a)',ERR=102,END=102) line
         IF (line.eq.' ') GO TO 103
c --- new format: lambda, t(lambda)
         IF (it.eq.1) THEN 
c read first line after the name of the first filter
            READ(81,'(a)',ERR=102,END=102) line
            DO i=1,kfil(1)
               j=0
               DO WHILE (line(1:1).ne.'#'.and.line.ne.' ')
                  j=j+1
                  READ(line,*) wh(j),res(j)
                  READ(81,'(a)',ERR=105,END=105) line
               END DO
c read first line of the following filter
               READ(81,'(a)',ERR=105,END=105) line
            END DO
 105        np(ifilt)=j
c --- old format: i,lambda,t(lambda)
         ELSE
            BACKSPACE 81
            DO i=1,kfil(1)
               READ(81,*) npp
               DO j=1,npp
                  READ(81,'(a)',ERR=102,END=102) line
                  READ(line,*) l,wh(j),res(j)
              END DO
            END DO
            np(ifilt)=npp
         END IF
         CLOSE(81)
c ----------------------------------------------------------------------
c filter ifilt is a convolution of different filters
      ELSE IF (nconvol.gt.1) THEN
c read filters
         DO k=1,nconvol
            CLOSE(81)
            OPEN(81,FILE=filtresfile,STATUS='old')
 104        READ(81,'(a)',ERR=102,END=102) line
            IF (line.eq.' ') GO TO 104
c --- new format: lambda, t(lambda)
            IF (it.eq.1) THEN 
               READ(81,'(a)',ERR=102,END=102) line
               DO i=1,kfil(k)
                  j=0
                  DO WHILE (line(1:1).ne.'#'.and.line.ne.' ')
                     j=j+1
                     READ(line,*) whc(k,j),resc(k,j)
                     READ(81,'(a)',ERR=106,END=106) line
                  END DO
                  READ(81,'(a)',ERR=106,END=106) line
               END DO
 106           npc(k)=j
c --- old format: i,lambda,t(lambda)
            ELSE
               BACKSPACE 81
               DO i=1,kfil(k)
                  READ(81,*) npp
                  DO j=1,npp
                     READ(81,'(a)',ERR=102,END=102) line
                     READ(line,*) l,whc(k,j),resc(k,j)
                  END DO
               END DO
               npc(k)=npp
            END IF
            CLOSE(81)
         END DO

c convolve filters: start from the first filter in the list
         DO i=1,npc(1)
            wh(i)=whc(1,i)
            res(i)=resc(1,i)
         END DO
         np(ifilt)=npc(1)

         DO k=2,nconvol
            wlmin=MAX(wh(1),whc(k,1))
            wlmax=MIN(wh(np(ifilt)),whc(k,npc(k)))
c first filter wh1 res1 from previous convolution (or the first one)
            DO i=1,np(ifilt)
               wh1(i)=wh(i)
               res1(i)=res(i)
            END DO
c select wavelengths of the convolved filter only in the common range
            j=0
            DO i=1,np(ifilt)
               IF (wh(i).ge.wlmin.and.wh(i).le.wlmax) THEN
                  j=j+1
                  wh(j)=wh1(i)
               END IF
            END DO

c add wavelength points of the filter k only in the common wavelength range
c (outside the wavelength intersection it is assumed to be res=0)
            DO i=1,npc(k)
               IF (whc(k,i).ge.wlmin.and.whc(k,i).le.wlmax) THEN
                  j=j+1
                  wh(j)=whc(k,i)
               END IF
c second filter is filter k
               wh2(i)=whc(k,i)
               res2(i)=resc(k,i)
            END DO
            
c sort all wavelength points
            CALL sort(j,wh)
c estimate the transmission functions of the 
            DO i=1,j
               l=0
               rr1=rlin(wh(i),wh1,res1,np(ifilt),l)
               l=0
               rr2=rlin(wh(i),wh2,res2,npc(k),l)
               res(i)=rr1*rr2
c               write(44,'(i3,1x,i4,f10.2,3(1x,ES12.5))') 
c     &              ifilt,i,wh(i),res(i),rr1,rr2
            END DO
            np(ifilt)=j
         END DO
      END IF
c ----------------------------------------------------------------------
c rebin if necessary
      IF (np(ifilt).le.mxwf) THEN
         DO j=1,np(ifilt)
            r(j,ifilt,1)=wh(j)
            r(j,ifilt,2)=res(j)
         END DO
      ELSE
         dlambda=wh(np(ifilt))-wh(1)
         dx=dlambda/DBLE(mxwf-1)
         DO j=1,mxwf
            r(j,ifilt,1)=wh(1)+(j-1)*dx
            l=0
            r(j,ifilt,2)=rlin(r(j,ifilt,1),wh,res,np(ifilt),l)
         END DO
         np(ifilt)=mxwf
      END IF
c ----------------------------------------------------------------------
c compute filter characteristics and modify according to iband
      rmax=0.0D0
      DO i=1,np(ifilt)
c photon response function: R_photon(lambda)=lambda/(hc)*R_energy(lambda)
         wh(i)=r(i,ifilt,1)
         IF (iband.eq.1) THEN
            res(i)=r(i,ifilt,2)*r(i,ifilt,1)
         ELSE
            res(i)=r(i,ifilt,2)
         END IF
         IF (res(i).gt.rmax) rmax=res(i)
      END DO
      DO i=1,np(ifilt)
         res(i)=res(i)/rmax
         r(i,ifilt,2)=res(i)
      END DO

c ... Surface and Effective wavelength
      surf=0.0D0
      conwl=0.0D0
      conwl2=0.0D0
      surf=trapz1(wh,res,np(ifilt)) ! Surface of the filter
      DO i=1,np(ifilt)
         aux(i)=wh(i)*res(i)
      END DO
      conwl=trapz1(wh,aux,np(ifilt))
      wleff=conwl/surf          ! Effective wavelength 
      DO i=1,np(ifilt)
         aux(i)=(wh(i)-wleff)*(wh(i)-wleff)*res(i)
      END DO
      conwl2=trapz1(wh,aux,np(ifilt))
      bandpass=2.D0*DSQRT(conwl2/surf) ! rms bandwidth Effective bandpass
      
c compute full width half maximum (you can pass it to the main program 
c replacing bandpass with fwhm in the subroutine arguments)
      i=1
      DO WHILE (res(i).le.0.5)
         rr1=wh(i)
         i=i+1
      END DO
      i=np(ifilt)
      DO WHILE (res(i).le.0.5)
         rr2=wh(i)
         i=i-1
      END DO
      fwhm=rr2-rr1              ! full width at half maximum

      RETURN
 102  STOP 'Error reading FILTERS_RES file'
      END 
